@@ -8,6 +8,7 @@ Demonstrates use of GLScatterPlotItem with rapidly-updating plots.
 import initExample
 
 import pyqtgraph as pg
+from pyqtgraph import functions as fn
 from pyqtgraph.Qt import QtCore, QtGui
 import pyqtgraph.opengl as gl
 import numpy as np
@@ -84,10 +85,10 @@ def update():
     global phase, sp2, d2
     s = -np.cos(d2*2+phase)
     color = np.empty((len(d2),4), dtype=np.float32)
-    color[:,3] = np.clip(s * 0.1, 0, 1)
-    color[:,0] = np.clip(s * 3.0, 0, 1)
-    color[:,1] = np.clip(s * 1.0, 0, 1)
-    color[:,2] = np.clip(s ** 3, 0, 1)
+    color[:,3] = fn.clip_array(s * 0.1, 0., 1.)
+    color[:,0] = fn.clip_array(s * 3.0, 0., 1.)
+    color[:,1] = fn.clip_array(s * 1.0, 0., 1.)
+    color[:,2] = fn.clip_array(s ** 3, 0., 1.)
     sp2.setData(color=color)
     phase -= 0.1
     
@@ -107,4 +108,4 @@ t.timeout.connect(update)
 t.start(50)
 
 if __name__ == '__main__':
-    pg.mkQApp().exec_()
+    pg.exec()
