@@ -47,4 +47,4 @@ timer.timeout.connect(update)
 timer.start(30)
     
 if __name__ == '__main__':
-    pg.mkQApp().exec_()
+    pg.exec()
