@@ -49,4 +49,4 @@ timer.timeout.connect(update)
 timer.start(0)
     
 if __name__ == '__main__':
-    pg.mkQApp().exec_()
+    pg.exec()
