@@ -18,7 +18,7 @@ import numpy as np
 from pyqtgraph.Qt import QtGui, QtCore
 import pyqtgraph as pg
 
-app = QtGui.QApplication([])
+app = pg.mkQApp("ViewBox Example")
 mw = QtGui.QMainWindow()
 mw.setWindowTitle('pyqtgraph example: ViewBox')
 mw.show()
@@ -42,13 +42,13 @@ class movableRect(QtGui.QGraphicsRectItem):
         self.setAcceptHoverEvents(True)
     def hoverEnterEvent(self, ev):
         self.savedPen = self.pen()
-        self.setPen(QtGui.QPen(QtGui.QColor(255, 255, 255)))
+        self.setPen(pg.mkPen(255, 255, 255))
         ev.ignore()
     def hoverLeaveEvent(self, ev):
         self.setPen(self.savedPen)
         ev.ignore()
     def mousePressEvent(self, ev):
-        if ev.button() == QtCore.Qt.LeftButton:
+        if ev.button() == QtCore.Qt.MouseButton.LeftButton:
             ev.accept()
             self.pressDelta = self.mapToParent(ev.pos()) - self.pos()
         else:
@@ -57,7 +57,7 @@ class movableRect(QtGui.QGraphicsRectItem):
         self.setPos(self.mapToParent(ev.pos()) - self.pressDelta)
         
 rect = movableRect(QtCore.QRectF(0, 0, 1, 1))
-rect.setPen(QtGui.QPen(QtGui.QColor(100, 200, 100)))
+rect.setPen(pg.mkPen(100, 200, 100))
 vb.addItem(rect)
 
 l.addItem(vb, 0, 1)
@@ -93,8 +93,5 @@ t = QtCore.QTimer()
 t.timeout.connect(updateData)
 t.start(50)
 
-## Start Qt event loop unless running in interactive mode.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.exec()
