@@ -23,7 +23,7 @@ plot.setWindowTitle('pyqtgraph example: text')
 curve = plot.plot(x,y)  ## add a single curve
 
 ## Create text object, use HTML tags to specify color/size
-text = pg.TextItem(html='<div style="text-align: center"><span style="color: #FFF;">This is the</span><br><span style="color: #FF0; font-size: 16pt;">PEAK</span></div>', anchor=(-0.3,1.3), border='w', fill=(0, 0, 255, 100))
+text = pg.TextItem(html='<div style="text-align: center"><span style="color: #FFF;">This is the</span><br><span style="color: #FF0; font-size: 16pt;">PEAK</span></div>', anchor=(-0.3,0.5), angle=45, border='w', fill=(0, 0, 255, 100))
 plot.addItem(text)
 text.setPos(0, y.max())
 
@@ -46,17 +46,11 @@ def update():
     global curvePoint, index
     index = (index + 1) % len(x)
     curvePoint.setPos(float(index)/(len(x)-1))
-    #text2.viewRangeChanged()
     text2.setText('[%0.1f, %0.1f]' % (x[index], y[index]))
     
 timer = QtCore.QTimer()
 timer.timeout.connect(update)
 timer.start(10)
 
-
-
-## Start Qt event loop unless running in interactive mode or using pyside.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.exec()
