@@ -3,7 +3,7 @@ import time
 import weakref
 import warnings
 
-from ..Qt import QtCore, QtGui
+from ..Qt import QtCore, QtGui, QT_LIB, isQObjectAlive
 from ..Point import Point
 from .. import functions as fn
 from .. import ptime as ptime
@@ -14,15 +14,9 @@ from .. import getConfigOption
 getMillis = lambda: int(round(time.time() * 1000))
 
 
-if hasattr(QtCore, 'PYQT_VERSION'):
-    try:
-        try:
-            from PyQt5 import sip
-        except ImportError:
-            import sip
-        HAVE_SIP = True
-    except ImportError:
-        HAVE_SIP = False
+if QT_LIB.startswith('PyQt'):
+    from ..Qt import sip
+    HAVE_SIP = True
 else:
     HAVE_SIP = False
 
@@ -122,7 +116,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
         self.lastHoverEvent = None
         self.minDragTime = 0.5  # drags shorter than 0.5 sec are interpreted as clicks
         
-        self.contextMenu = [QtGui.QAction("Export...", self)]
+        self.contextMenu = [QtGui.QAction(QtCore.QCoreApplication.translate("GraphicsScene", "Export..."), self)]
         self.contextMenu[0].triggered.connect(self.showExportDialog)
         
         self.exportDialog = None
@@ -158,7 +152,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
         self._moveDistance = d
 
     def mousePressEvent(self, ev):
-        QtGui.QGraphicsScene.mousePressEvent(self, ev)
+        super().mousePressEvent(ev)
         if self.mouseGrabberItem() is None:  ## nobody claimed press; we are free to generate drag/click events
             if self.lastHoverEvent is not None:
                 # If the mouse has moved since the last hover event, send a new one.
@@ -171,18 +165,18 @@ class GraphicsScene(QtGui.QGraphicsScene):
             ## set focus on the topmost focusable item under this click
             items = self.items(ev.scenePos())
             for i in items:
-                if i.isEnabled() and i.isVisible() and int(i.flags() & i.ItemIsFocusable) > 0:
-                    i.setFocus(QtCore.Qt.MouseFocusReason)
+                if i.isEnabled() and i.isVisible() and (i.flags() & i.GraphicsItemFlag.ItemIsFocusable):
+                    i.setFocus(QtCore.Qt.FocusReason.MouseFocusReason)
                     break
 
     def _moveEventIsAllowed(self):
         # For ignoring events that are too close together
-        
+
         # Max number of events per second
         rateLimit = getConfigOption('mouseRateLimit')
         if rateLimit <= 0:
             return True
-        
+
         # Delay between events (in milliseconds)
         delay = 1000.0 / rateLimit
         if getMillis() - self._lastMoveEventTime >= delay:
@@ -197,35 +191,35 @@ class GraphicsScene(QtGui.QGraphicsScene):
             self.sigMouseMoved.emit(ev.scenePos())
 
             # First allow QGraphicsScene to eliver hoverEvent/Move/Exit Events
-            QtGui.QGraphicsScene.mouseMoveEvent(self, ev)
+            super().mouseMoveEvent(ev)
             # Next Deliver our own Hover Events
             self.sendHoverEvents(ev)
-            if int(ev.buttons()) != 0:
+            if ev.buttons():
                 # button is pressed' send mouseMoveEvents and mouseDragEvents
-                QtGui.QGraphicsScene.mouseMoveEvent(self, ev)
+                super().mouseMoveEvent(ev)
                 if self.mouseGrabberItem() is None:
                     now = ptime.time()
                     init = False
                     ## keep track of which buttons are involved in dragging
-                    for btn in [QtCore.Qt.LeftButton, QtCore.Qt.MidButton, QtCore.Qt.RightButton]:
-                        if int(ev.buttons() & btn) == 0:
+                    for btn in [QtCore.Qt.MouseButton.LeftButton, QtCore.Qt.MouseButton.MiddleButton, QtCore.Qt.MouseButton.RightButton]:
+                        if not (ev.buttons() & btn):
                             continue
-                        if int(btn) not in self.dragButtons:  ## see if we've dragged far enough yet
-                            cev = [e for e in self.clickEvents if int(e.button()) == int(btn)]
+                        if btn not in self.dragButtons:  ## see if we've dragged far enough yet
+                            cev = [e for e in self.clickEvents if e.button() == btn]
                             if cev:
                                 cev = cev[0]
                                 dist = Point(ev.scenePos() - cev.scenePos()).length()
                                 if dist == 0 or (dist < self._moveDistance and now - cev.time() < self.minDragTime):
                                     continue
                                 init = init or (len(self.dragButtons) == 0)  ## If this is the first button to be dragged, then init=True
-                                self.dragButtons.append(int(btn))
+                                self.dragButtons.append(btn)
                     ## if we have dragged buttons, deliver a drag event
                     if len(self.dragButtons) > 0:
                         if self.sendDragEvent(ev, init=init):
                             ev.accept()
 
         else:
-            QtGui.QGraphicsScene.mouseMoveEvent(self, ev)
+            super().mouseMoveEvent(ev)
             # if you do not accept event (which is ignored) then cursor will disappear
             ev.accept()
                 
@@ -241,24 +235,24 @@ class GraphicsScene(QtGui.QGraphicsScene):
                     ev.accept()
                 self.dragButtons.remove(ev.button())
             else:
-                cev = [e for e in self.clickEvents if int(e.button()) == int(ev.button())]
+                cev = [e for e in self.clickEvents if e.button() == ev.button()]
                 if cev:
                     if self.sendClickEvent(cev[0]):
                         #print "sent click event"
                         ev.accept()
                     self.clickEvents.remove(cev[0])
                 
-        if int(ev.buttons()) == 0:
+        if not ev.buttons():
             self.dragItem = None
             self.dragButtons = []
             self.clickEvents = []
             self.lastDrag = None
-        QtGui.QGraphicsScene.mouseReleaseEvent(self, ev)
+        super().mouseReleaseEvent(ev)
         
         self.sendHoverEvents(ev)  ## let items prepare for next click/drag
 
     def mouseDoubleClickEvent(self, ev):
-        QtGui.QGraphicsScene.mouseDoubleClickEvent(self, ev)
+        super().mouseDoubleClickEvent(ev)
         if self.mouseGrabberItem() is None:  ## nobody claimed press; we are free to generate drag/click events
             self.clickEvents.append(MouseClickEvent(ev, double=True))
         
@@ -270,7 +264,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
             items = []
             event = HoverEvent(None, acceptable)
         else:
-            acceptable = int(ev.buttons()) == 0  ## if we are in mid-drag, do not allow items to accept the hover event.
+            acceptable = not ev.buttons()  ## if we are in mid-drag, do not allow items to accept the hover event.
             event = HoverEvent(ev, acceptable)
             items = self.itemsNearEvent(event, hoverable=True)
             self.sigMouseHover.emit(items)
@@ -298,7 +292,9 @@ class GraphicsScene(QtGui.QGraphicsScene):
         for item in prevItems:
             event.currentItem = item
             try:
-                if item.scene() is self:
+                # NOTE: isQObjectAlive(item) was added for PySide6 where
+                #       verlet_chain_demo.py triggers a RuntimeError.
+                if isQObjectAlive(item) and item.scene() is self:
                     item.hoverEvent(event)
             except:
                 debug.printExc("Error sending hover exit event:")
@@ -308,9 +304,9 @@ class GraphicsScene(QtGui.QGraphicsScene):
         # Update last hover event unless:
         #   - mouse is dragging (move+buttons); in this case we want the dragged
         #     item to continue receiving events until the drag is over
-        #   - event is not a mouse event (QEvent.Leave sometimes appears here)
-        if (ev.type() == ev.GraphicsSceneMousePress or 
-            (ev.type() == ev.GraphicsSceneMouseMove and int(ev.buttons()) == 0)):
+        #   - event is not a mouse event (QEvent.Type.Leave sometimes appears here)
+        if (ev.type() == ev.Type.GraphicsSceneMousePress or 
+            (ev.type() == ev.Type.GraphicsSceneMouseMove and not ev.buttons())):
             self.lastHoverEvent = event  ## save this so we can ask about accepted events later.
 
     def sendDragEvent(self, ev, init=False, final=False):
@@ -348,8 +344,8 @@ class GraphicsScene(QtGui.QGraphicsScene):
                         if event.isAccepted():
                             #print "   --> accepted"
                             self.dragItem = item
-                            if int(item.flags() & item.ItemIsFocusable) > 0:
-                                item.setFocus(QtCore.Qt.MouseFocusReason)
+                            if item.flags() & item.GraphicsItemFlag.ItemIsFocusable:
+                                item.setFocus(QtCore.Qt.FocusReason.MouseFocusReason)
                             break
         elif self.dragItem is not None:
             event.currentItem = self.dragItem
@@ -393,8 +389,8 @@ class GraphicsScene(QtGui.QGraphicsScene):
                             debug.printExc("Error sending click event:")
                             
                         if ev.isAccepted():
-                            if int(item.flags() & item.ItemIsFocusable) > 0:
-                                item.setFocus(QtCore.Qt.MouseFocusReason)
+                            if item.flags() & item.GraphicsItemFlag.ItemIsFocusable:
+                                item.setFocus(QtCore.Qt.FocusReason.MouseFocusReason)
                             break
         self.sigMouseClicked.emit(ev)
         return ev.isAccepted()
@@ -423,7 +419,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
         item = QtGui.QGraphicsScene.itemAt(self, *args)
         return self.translateGraphicsItem(item)
 
-    def itemsNearEvent(self, event, selMode=QtCore.Qt.IntersectsItemShape, sortOrder=QtCore.Qt.DescendingOrder, hoverable=False):
+    def itemsNearEvent(self, event, selMode=QtCore.Qt.ItemSelectionMode.IntersectsItemShape, sortOrder=QtCore.Qt.SortOrder.DescendingOrder, hoverable=False):
         """
         Return an iterator that iterates first through the items that directly intersect point (in Z order)
         followed by any other items that are within the scene's click radius.
