@@ -1,10 +1,37 @@
+import warnings
+from math import hypot
+from collections import OrderedDict
+from functools import reduce
 from ..Qt import QtGui, QtCore, isQObjectAlive
 from ..GraphicsScene import GraphicsScene
 from ..Point import Point
 from .. import functions as fn
 import weakref
 import operator
-from ..util.lru_cache import LRUCache
+
+__all__ = ['GraphicsItem']
+
+# Recipe from https://docs.python.org/3.8/library/collections.html#collections.OrderedDict
+# slightly adapted for Python 3.7 compatibility
+class LRU(OrderedDict):
+    'Limit size, evicting the least recently looked-up key when full'
+
+    def __init__(self, maxsize=128, *args, **kwds):
+        self.maxsize = maxsize
+        super().__init__(*args, **kwds)
+
+    def __getitem__(self, key):
+        value = super().__getitem__(key)
+        self.move_to_end(key)
+        return value
+
+    def __setitem__(self, key, value):
+        if key in self:
+            self.move_to_end(key)
+        super().__setitem__(key, value)
+        if len(self) > self.maxsize:
+            oldest = next(iter(self))
+            del self[oldest]
 
 
 class GraphicsItem(object):
@@ -18,9 +45,9 @@ class GraphicsItem(object):
 
     The GraphicsView system places a lot of emphasis on the notion that the graphics within the scene should be device independent--you should be able to take the same graphics and display them on screens of different resolutions, printers, export to SVG, etc. This is nice in principle, but causes me a lot of headache in practice. It means that I have to circumvent all the device-independent expectations any time I want to operate in pixel coordinates rather than arbitrary scene coordinates. A lot of the code in GraphicsItem is devoted to this task--keeping track of view widgets and device transforms, computing the size and shape of a pixel in local item coordinates, etc. Note that in item coordinates, a pixel does not have to be square or even rectangular, so just asking how to increase a bounding rect by 2px can be a rather complex task.
     """
-    _pixelVectorGlobalCache = LRUCache(100, 70)
-    
-    def __init__(self, register=True):
+    _pixelVectorGlobalCache = LRU(100)
+
+    def __init__(self, register=None):
         if not hasattr(self, '_qtBaseClass'):
             for b in self.__class__.__bases__:
                 if issubclass(b, QtGui.QGraphicsItem):
@@ -28,18 +55,19 @@ class GraphicsItem(object):
                     break
         if not hasattr(self, '_qtBaseClass'):
             raise Exception('Could not determine Qt base class for GraphicsItem: %s' % str(self))
-        
+
         self._pixelVectorCache = [None, None]
         self._viewWidget = None
         self._viewBox = None
         self._connectedView = None
         self._exportOpts = False   ## If False, not currently exporting. Otherwise, contains dict of export options.
-        if register:
-            GraphicsScene.registerObject(self)  ## workaround for pyqt bug in graphicsscene.items()
-                    
+        self._cachedView = None
+        if register is not None and register:
+            warnings.warn(
+                "'register' argument is deprecated and does nothing, will be removed in 0.13",
+                DeprecationWarning, stacklevel=2
+            )
 
-                    
-                    
     def getViewWidget(self):
         """
         Return the view widget for this item. 
@@ -95,15 +123,11 @@ class GraphicsItem(object):
     def forgetViewBox(self):
         self._viewBox = None
         
-        
     def deviceTransform(self, viewportTransform=None):
         """
         Return the transform that converts local item coordinates to device coordinates (usually pixels).
         Extends deviceTransform to automatically determine the viewportTransform.
         """
-        if self._exportOpts is not False and 'painter' in self._exportOpts: ## currently exporting; device transform may be different.
-            return self._exportOpts['painter'].deviceTransform() * self.sceneTransform()
-            
         if viewportTransform is None:
             view = self.getViewWidget()
             if view is None:
@@ -145,12 +169,17 @@ class GraphicsItem(object):
             p = p.parentItem()
             if p is None:
                 break
-            if p.flags() & self.ItemClipsChildrenToShape:
+            if p.flags() & self.GraphicsItemFlag.ItemClipsChildrenToShape:
                 parents.append(p)
         return parents
     
     def viewRect(self):
-        """Return the bounds (in item coordinates) of this item's ViewBox or GraphicsWidget"""
+        """Return the visible bounds of this item's ViewBox or GraphicsWidget,
+        in the local coordinate system of the item."""
+        if self._cachedView is not None:
+            return self._cachedView
+
+        # Note that in cases of early returns here, the view cache stays empty (None).
         view = self.getViewBox()
         if view is None:
             return None
@@ -160,10 +189,12 @@ class GraphicsItem(object):
 
         bounds = bounds.normalized()
         
+        self._cachedView = bounds
+            
         ## nah.
         #for p in self.getBoundingParents():
             #bounds &= self.mapRectFromScene(p.sceneBoundingRect())
-            
+
         return bounds
         
         
@@ -187,24 +218,23 @@ class GraphicsItem(object):
         ## (such as when looking at unix timestamps), we can get floating-point errors.
         dt.setMatrix(dt.m11(), dt.m12(), 0, dt.m21(), dt.m22(), 0, 0, 0, 1)
         
+        if direction is None:
+            direction = QtCore.QPointF(1, 0)
+        elif direction.manhattanLength() == 0:
+            raise Exception("Cannot compute pixel length for 0-length vector.")
+
+        key = (dt.m11(), dt.m21(), dt.m12(), dt.m22(), direction.x(), direction.y())
+
         ## check local cache
-        if direction is None and dt == self._pixelVectorCache[0]:
+        if key == self._pixelVectorCache[0]:
             return tuple(map(Point, self._pixelVectorCache[1]))  ## return a *copy*
-        
+
         ## check global cache
-        #key = (dt.m11(), dt.m21(), dt.m31(), dt.m12(), dt.m22(), dt.m32(), dt.m31(), dt.m32())
-        key = (dt.m11(), dt.m21(), dt.m12(), dt.m22())
         pv = self._pixelVectorGlobalCache.get(key, None)
-        if direction is None and pv is not None:
-            self._pixelVectorCache = [dt, pv]
+        if pv is not None:
+            self._pixelVectorCache = [key, pv]
             return tuple(map(Point,pv))  ## return a *copy*
         
-        
-        if direction is None:
-            direction = QtCore.QPointF(1, 0)  
-        if direction.manhattanLength() == 0:
-            raise Exception("Cannot compute pixel length for 0-length vector.")
-            
         ## attempt to re-scale direction vector to fit within the precision of the coordinate system
         ## Here's the problem: we need to map the vector 'direction' from the item to the device, via transform 'dt'.
         ## In some extreme cases, this mapping can fail unless the length of 'direction' is cleverly chosen.
@@ -280,7 +310,7 @@ class GraphicsItem(object):
         v = self.pixelVectors()
         if v == (None, None):
             return None, None
-        return (v[0].x()**2+v[0].y()**2)**0.5, (v[1].x()**2+v[1].y()**2)**0.5
+        return (hypot(v[0].x(), v[0].y()), hypot(v[1].x(), v[1].y()))  # lengths
 
     def pixelWidth(self):
         ## deprecated
@@ -409,19 +439,16 @@ class GraphicsItem(object):
         """
         if relativeItem is None:
             relativeItem = self.parentItem()
-            
 
         tr = self.itemTransform(relativeItem)
         if isinstance(tr, tuple):  ## difference between pyside and pyqt
             tr = tr[0]
-        #vec = tr.map(Point(1,0)) - tr.map(Point(0,0))
         vec = tr.map(QtCore.QLineF(0,0,1,0))
-        #return Point(vec).angle(Point(1,0))
         return vec.angleTo(QtCore.QLineF(vec.p1(), vec.p1()+QtCore.QPointF(1,0)))
         
     #def itemChange(self, change, value):
         #ret = self._qtBaseClass.itemChange(self, change, value)
-        #if change == self.ItemParentHasChanged or change == self.ItemSceneHasChanged:
+        #if change == self.GraphicsItemChange.ItemParentHasChanged or change == self.ItemSceneHasChanged:
             #print "Item scene changed:", self
             #self.setChildScene(self)  ## This is bizarre.
         #return ret
@@ -448,6 +475,10 @@ class GraphicsItem(object):
         ## called to see whether this item has a new view to connect to
         ## NOTE: This is called from GraphicsObject.itemChange or GraphicsWidget.itemChange.
 
+        if not hasattr(self, '_connectedView'):
+            # Happens when Python is shutting down.
+            return
+
         ## It is possible this item has moved to a different ViewBox or widget;
         ## clear out previously determined references to these.
         self.forgetViewBox()
@@ -523,14 +554,17 @@ class GraphicsItem(object):
         """
         Called whenever the view coordinates of the ViewBox containing this item have changed.
         """
+        # when this is called, _cachedView is not invalidated.
+        # this means that for functions overriding viewRangeChanged, viewRect() may be stale.
         pass
     
     def viewTransformChanged(self):
         """
         Called whenever the transformation matrix of the view has changed.
         (eg, the view range has changed or the view was resized)
+        Invalidates the viewRect cache.
         """
-        pass
+        self._cachedView = None
     
     #def prepareGeometryChange(self):
         #self._qtBaseClass.prepareGeometryChange(self)
