@@ -1,58 +1,108 @@
-from ..Qt import QtCore, QtGui, QtOpenGL
+from ..Qt import QtCore, QtGui, QtWidgets
 from OpenGL.GL import *
 import OpenGL.GL.framebufferobjects as glfbo
 import numpy as np
 from .. import Vector
 from .. import functions as fn
-
+import warnings
+from math import cos, sin, tan, radians
 ##Vector = QtGui.QVector3D
 
-ShareWidget = None
 
-class GLViewWidget(QtOpenGL.QGLWidget):
-    """
-    Basic widget for displaying 3D data
+class GLViewWidget(QtWidgets.QOpenGLWidget):
+    
+    def __init__(self, parent=None, devicePixelRatio=None, rotationMethod='euler'):
+        """    
+        Basic widget for displaying 3D data
         - Rotation/scale controls
         - Axis/grid display
         - Export options
 
-    """
-    
-    def __init__(self, parent=None):
-        global ShareWidget
+        ================ ==============================================================
+        **Arguments:**
+        parent           (QObject, optional): Parent QObject. Defaults to None.
+        devicePixelRatio (float, optional):  High-DPI displays Qt5 should automatically
+                         detect the correct resolution. For Qt4, specify the 
+                         ``devicePixelRatio`` argument when initializing the widget 
+                         (usually this value is 1-2). Defaults to None.
+        rotationMethod   (str): Mechanimsm to drive the rotation method, options are 
+                         'euler' and 'quaternion'. Defaults to 'euler'.
+        ================ ==============================================================
+        """
 
-        if ShareWidget is None:
-            ## create a dummy widget to allow sharing objects (textures, shaders, etc) between views
-            ShareWidget = QtOpenGL.QGLWidget()
-            
-        QtOpenGL.QGLWidget.__init__(self, parent, ShareWidget)
+        QtWidgets.QOpenGLWidget.__init__(self, parent)
         
-        self.setFocusPolicy(QtCore.Qt.ClickFocus)
+        self.setFocusPolicy(QtCore.Qt.FocusPolicy.ClickFocus)
+
+        if rotationMethod not in {"euler", "quaternion"}:
+            raise RuntimeError("Rotation method should be either 'euler' or 'quaternion'")
         
         self.opts = {
             'center': Vector(0,0,0),  ## will always appear at the center of the widget
+            'rotation' : QtGui.QQuaternion(1,0,0,0), ## camera rotation (quaternion:wxyz)
             'distance': 10.0,         ## distance of camera from center
             'fov':  60,               ## horizontal field of view in degrees
-            'elevation':  30,         ## camera's angle of elevation in degrees
-            'azimuth': 45,            ## camera's azimuthal angle in degrees 
-                                      ## (rotation around z-axis 0 points along x-axis)
+            'elevation': 30,          ## camera's angle of elevation in degrees
+            'azimuth': 45,            ## camera's azimuthal angle in degrees 	
+                                      ## (rotation around z-axis 0 points along x-axis)	
             'viewport': None,         ## glViewport params; None == whole widget
+            'devicePixelRatio': devicePixelRatio,
+            'rotationMethod': rotationMethod
         }
-        self.setBackgroundColor('k')
+        self.reset()
         self.items = []
-        self.noRepeatKeys = [QtCore.Qt.Key_Right, QtCore.Qt.Key_Left, QtCore.Qt.Key_Up, QtCore.Qt.Key_Down, QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]
+        
+        self.noRepeatKeys = [QtCore.Qt.Key.Key_Right, QtCore.Qt.Key.Key_Left, QtCore.Qt.Key.Key_Up, QtCore.Qt.Key.Key_Down, QtCore.Qt.Key.Key_PageUp, QtCore.Qt.Key.Key_PageDown]
         self.keysPressed = {}
         self.keyTimer = QtCore.QTimer()
         self.keyTimer.timeout.connect(self.evalKeyState)
-        
         self.makeCurrent()
 
+
+    def _updateScreen(self, screen):
+        self._updatePixelRatio()
+        if screen is not None:
+            screen.physicalDotsPerInchChanged.connect(self._updatePixelRatio)
+            screen.logicalDotsPerInchChanged.connect(self._updatePixelRatio)
+    
+    def _updatePixelRatio(self):
+        event = QtGui.QResizeEvent(self.size(), self.size())
+        self.resizeEvent(event)
+    
+    def showEvent(self, event):
+        window = self.window().windowHandle()
+        window.screenChanged.connect(self._updateScreen)
+        self._updateScreen(window.screen())
+        
+    def width(self):
+        dpr = self.devicePixelRatio()
+        return int(super().width() * dpr)
+    
+    def height(self):
+        dpr = self.devicePixelRatio()
+        return int(super().height() * dpr)
+
+
+    def reset(self):
+        """
+        Initialize the widget state or reset the current state to the original state.
+        """
+        self.opts['center'] = Vector(0,0,0)  ## will always appear at the center of the widget
+        self.opts['distance'] = 10.0         ## distance of camera from center
+        self.opts['fov'] = 60                ## horizontal field of view in degrees
+        self.opts['elevation'] = 30          ## camera's angle of elevation in degrees
+        self.opts['azimuth'] = 45            ## camera's azimuthal angle in degrees 
+                                             ## (rotation around z-axis 0 points along x-axis)
+        self.opts['viewport'] = None         ## glViewport params; None == whole widget
+        self.setBackgroundColor('k')        
+
     def addItem(self, item):
         self.items.append(item)
-        if hasattr(item, 'initializeGL'):
+
+        if self.isValid():
             self.makeCurrent()
             try:
-                item.initializeGL()
+                item.initialize()
             except:
                 self.checkOpenGLVersion('Error while adding item %s to GLViewWidget.' % str(item))
                 
@@ -61,20 +111,36 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         self.update()
         
     def removeItem(self, item):
+        """
+        Remove the item from the scene.
+        """
         self.items.remove(item)
         item._setView(None)
         self.update()
-        
+
+    def clear(self):
+        """
+        Remove all items from the scene.
+        """
+        for item in self.items:
+            item._setView(None)
+        self.items = []
+        self.update()        
         
     def initializeGL(self):
-        self.resizeGL(self.width(), self.height())
+        """
+        Initialize items that were not initialized during addItem().
+        """
+        for item in self.items:
+            if not item.isInitialized():
+                item.initialize()
         
     def setBackgroundColor(self, *args, **kwds):
         """
         Set the background color of the widget. Accepts the same arguments as
-        pg.mkColor().
+        pg.mkColor() and pg.glColor().
         """
-        self.opts['bgcolor'] = fn.mkColor(*args, **kwds)
+        self.opts['bgcolor'] = fn.glColor(*args, **kwds)
         self.update()
         
     def getViewport(self):
@@ -82,7 +148,15 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         if vp is None:
             return (0, 0, self.width(), self.height())
         else:
-            return vp
+            dpr = self.devicePixelRatio()
+            return tuple([int(x * dpr) for x in vp])
+        
+    def devicePixelRatio(self):
+        dpr = self.opts['devicePixelRatio']
+        if dpr is not None:
+            return dpr
+        
+        return self.devicePixelRatioF()
         
     def resizeGL(self, w, h):
         pass
@@ -97,7 +171,6 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         glMultMatrixf(a.transpose())
 
     def projectionMatrix(self, region=None):
-        # Xw = (Xnd + 1) * width/2 + X
         if region is None:
             region = (0, 0, self.width(), self.height())
         
@@ -107,11 +180,9 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         nearClip = dist * 0.001
         farClip = dist * 1000.
 
-        r = nearClip * np.tan(fov * 0.5 * np.pi / 180.)
+        r = nearClip * tan(0.5 * radians(fov))
         t = r * h / w
 
-        # convert screen coordinates (region) to normalized device coordinates
-        # Xnd = (Xw - X0) * 2/width - 1
         ## Note that X0 and width in these equations must be the values used in viewport
         left  = r * ((region[0]-x0) * (2.0/w) - 1)
         right = r * ((region[0]+region[2]-x0) * (2.0/w) - 1)
@@ -132,8 +203,12 @@ class GLViewWidget(QtOpenGL.QGLWidget):
     def viewMatrix(self):
         tr = QtGui.QMatrix4x4()
         tr.translate( 0.0, 0.0, -self.opts['distance'])
-        tr.rotate(self.opts['elevation']-90, 1, 0, 0)
-        tr.rotate(self.opts['azimuth']+90, 0, 0, -1)
+        if self.opts['rotationMethod'] == 'quaternion':
+            tr.rotate(self.opts['rotation'])
+        else:
+            # default rotation method
+            tr.rotate(self.opts['elevation']-90, 1, 0, 0)
+            tr.rotate(self.opts['azimuth']+90, 0, 0, -1)  
         center = self.opts['center']
         tr.translate(-center.x(), -center.y(), -center.z())
         return tr
@@ -174,7 +249,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         self.setProjection(region=region)
         self.setModelview()
         bgcolor = self.opts['bgcolor']
-        glClearColor(bgcolor.red(), bgcolor.green(), bgcolor.blue(), 1.0)
+        glClearColor(*bgcolor)
         glClear( GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT )
         self.drawItemTree(useItemNames=useItemNames)
         
@@ -221,62 +296,128 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                     glMatrixMode(GL_MODELVIEW)
                     glPopMatrix()
             
-    def setCameraPosition(self, pos=None, distance=None, elevation=None, azimuth=None):
+    def setCameraPosition(self, pos=None, distance=None, elevation=None, azimuth=None, rotation=None):
+        if pos is not None:
+            self.opts['center'] = pos
         if distance is not None:
             self.opts['distance'] = distance
-        if elevation is not None:
-            self.opts['elevation'] = elevation
-        if azimuth is not None:
-            self.opts['azimuth'] = azimuth
+        if rotation is not None:
+            # set with quaternion
+            self.opts['rotation'] = rotation
+        else:
+            # set with elevation-azimuth, restored for compatibility
+            eu = self.opts['rotation'].toEulerAngles()
+            if azimuth is not None:
+                eu.setZ(-azimuth-90)
+            if elevation is not None:
+                eu.setX(elevation-90)
+            self.opts['rotation'] = QtGui.QQuaternion.fromEulerAngles(eu)
         self.update()
         
-        
-        
     def cameraPosition(self):
         """Return current position of camera based on center, dist, elevation, and azimuth"""
         center = self.opts['center']
         dist = self.opts['distance']
-        elev = self.opts['elevation'] * np.pi/180.
-        azim = self.opts['azimuth'] * np.pi/180.
-        
-        pos = Vector(
-            center.x() + dist * np.cos(elev) * np.cos(azim),
-            center.y() + dist * np.cos(elev) * np.sin(azim),
-            center.z() + dist * np.sin(elev)
-        )
-        
+        if self.opts['rotationMethod'] == "quaternion":
+            pos = Vector(center - self.opts['rotation'].rotatedVector(Vector(0,0,dist) ))
+        else:
+            # using 'euler' rotation method
+            elev = radians(self.opts['elevation'])
+            azim = radians(self.opts['azimuth'])
+            pos = Vector(
+                center.x() + dist * cos(elev) * cos(azim),
+                center.y() + dist * cos(elev) * sin(azim),
+                center.z() + dist * sin(elev)
+            )
         return pos
 
     def orbit(self, azim, elev):
         """Orbits the camera around the center position. *azim* and *elev* are given in degrees."""
-        self.opts['azimuth'] += azim
-        #self.opts['elevation'] += elev
-        self.opts['elevation'] = np.clip(self.opts['elevation'] + elev, -90, 90)
+        if self.opts['rotationMethod'] == 'quaternion':
+            q = QtGui.QQuaternion.fromEulerAngles(
+                    elev, -azim, 0
+                    ) # rx-ry-rz
+            q *= self.opts['rotation']
+            self.opts['rotation'] = q
+        else: # default euler rotation method
+            self.opts['azimuth'] += azim
+            self.opts['elevation'] = fn.clip_scalar(self.opts['elevation'] + elev, -90., 90.)
         self.update()
         
-    def pan(self, dx, dy, dz, relative=False):
+    def pan(self, dx, dy, dz, relative='global'):
         """
         Moves the center (look-at) position while holding the camera in place. 
         
-        If relative=True, then the coordinates are interpreted such that x
-        if in the global xy plane and points to the right side of the view, y is
-        in the global xy plane and orthogonal to x, and z points in the global z
-        direction. Distances are scaled roughly such that a value of 1.0 moves
+        ==============  =======================================================
+        **Arguments:**
+        *dx*            Distance to pan in x direction
+        *dy*            Distance to pan in y direction
+        *dz*            Distance to pan in z direction
+        *relative*      String that determines the direction of dx,dy,dz. 
+                        If "global", then the global coordinate system is used.
+                        If "view", then the z axis is aligned with the view
+                        direction, and x and y axes are in the plane of the
+                        view: +x points right, +y points up. 
+                        If "view-upright", then x is in the global xy plane and
+                        points to the right side of the view, y is in the
+                        global xy plane and orthogonal to x, and z points in
+                        the global z direction.
+        ==============  =======================================================
+        
+        Distances are scaled roughly such that a value of 1.0 moves
         by one pixel on screen.
         
+        Prior to version 0.11, *relative* was expected to be either True (x-aligned) or
+        False (global). These values are deprecated but still recognized.
         """
-        if not relative:
+        # for backward compatibility:
+        if isinstance(relative, bool):
+            warnings.warn(
+                "'relative' as a boolean is deprecated, and will not be recognized in 0.13. "
+                "Acceptable values are 'global', 'view', or 'view-upright'",
+                DeprecationWarning, stacklevel=2
+            )    
+        relative = {True: "view-upright", False: "global"}.get(relative, relative)
+        if relative == 'global':
             self.opts['center'] += QtGui.QVector3D(dx, dy, dz)
-        else:
+        elif relative == 'view-upright':
             cPos = self.cameraPosition()
             cVec = self.opts['center'] - cPos
             dist = cVec.length()  ## distance from camera to center
-            xDist = dist * 2. * np.tan(0.5 * self.opts['fov'] * np.pi / 180.)  ## approx. width of view at distance of center point
+            xDist = dist * 2. * tan(0.5 * radians(self.opts['fov']))  ## approx. width of view at distance of center point
             xScale = xDist / self.width()
             zVec = QtGui.QVector3D(0,0,1)
             xVec = QtGui.QVector3D.crossProduct(zVec, cVec).normalized()
             yVec = QtGui.QVector3D.crossProduct(xVec, zVec).normalized()
             self.opts['center'] = self.opts['center'] + xVec * xScale * dx + yVec * xScale * dy + zVec * xScale * dz
+        elif relative == 'view':
+            # pan in plane of camera
+
+            if self.opts['rotationMethod'] == 'quaternion':
+                # obtain basis vectors
+                qc = self.opts['rotation'].conjugated()
+                xv = qc.rotatedVector( Vector(1,0,0) )
+                yv = qc.rotatedVector( Vector(0,1,0) )
+                zv = qc.rotatedVector( Vector(0,0,1) )
+
+                scale_factor = self.pixelSize( self.opts['center'] )
+
+                # apply translation
+                self.opts['center'] += scale_factor * (xv*-dx + yv*dy + zv*dz)
+            else: # use default euler rotation method
+                elev = radians(self.opts['elevation'])
+                azim = radians(self.opts['azimuth'])
+                fov = radians(self.opts['fov'])
+                dist = (self.opts['center'] - self.cameraPosition()).length()
+                fov_factor = tan(fov / 2) * 2
+                scale_factor = dist * fov_factor / self.width()
+                z = scale_factor * cos(elev) * dy
+                x = scale_factor * (sin(azim) * dx - sin(elev) * cos(azim) * dy)
+                y = scale_factor * (cos(azim) * dx + sin(elev) * sin(azim) * dy)
+                self.opts['center'] += QtGui.QVector3D(x, -y, z)
+        else:
+            raise ValueError("relative argument must be global, view, or view-upright")
+        
         self.update()
         
     def pixelSize(self, pos):
@@ -290,24 +431,28 @@ class GLViewWidget(QtOpenGL.QGLWidget):
             dist = ((pos-cam)**2).sum(axis=-1)**0.5
         else:
             dist = (pos-cam).length()
-        xDist = dist * 2. * np.tan(0.5 * self.opts['fov'] * np.pi / 180.)
+        xDist = dist * 2. * tan(0.5 * radians(self.opts['fov']))
         return xDist / self.width()
         
     def mousePressEvent(self, ev):
-        self.mousePos = ev.pos()
+        lpos = ev.position() if hasattr(ev, 'position') else ev.localPos()
+        self.mousePos = lpos
         
     def mouseMoveEvent(self, ev):
-        diff = ev.pos() - self.mousePos
-        self.mousePos = ev.pos()
+        lpos = ev.position() if hasattr(ev, 'position') else ev.localPos()
+        diff = lpos - self.mousePos
+        self.mousePos = lpos
         
-        if ev.buttons() == QtCore.Qt.LeftButton:
-            self.orbit(-diff.x(), diff.y())
-            #print self.opts['azimuth'], self.opts['elevation']
-        elif ev.buttons() == QtCore.Qt.MidButton:
-            if (ev.modifiers() & QtCore.Qt.ControlModifier):
-                self.pan(diff.x(), 0, diff.y(), relative=True)
+        if ev.buttons() == QtCore.Qt.MouseButton.LeftButton:
+            if (ev.modifiers() & QtCore.Qt.KeyboardModifier.ControlModifier):
+                self.pan(diff.x(), diff.y(), 0, relative='view')
             else:
-                self.pan(diff.x(), diff.y(), 0, relative=True)
+                self.orbit(-diff.x(), diff.y())
+        elif ev.buttons() == QtCore.Qt.MouseButton.MiddleButton:
+            if (ev.modifiers() & QtCore.Qt.KeyboardModifier.ControlModifier):
+                self.pan(diff.x(), 0, diff.y(), relative='view-upright')
+            else:
+                self.pan(diff.x(), diff.y(), 0, relative='view-upright')
         
     def mouseReleaseEvent(self, ev):
         pass
@@ -322,12 +467,14 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         #self.paintGL(region=region)
         #self.swapBuffers()
         
-        
     def wheelEvent(self, ev):
-        if (ev.modifiers() & QtCore.Qt.ControlModifier):
-            self.opts['fov'] *= 0.999**ev.delta()
+        delta = ev.angleDelta().x()
+        if delta == 0:
+            delta = ev.angleDelta().y()
+        if (ev.modifiers() & QtCore.Qt.KeyboardModifier.ControlModifier):
+            self.opts['fov'] *= 0.999**delta
         else:
-            self.opts['distance'] *= 0.999**ev.delta()
+            self.opts['distance'] *= 0.999**delta
         self.update()
 
     def keyPressEvent(self, ev):
@@ -353,34 +500,52 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         speed = 2.0
         if len(self.keysPressed) > 0:
             for key in self.keysPressed:
-                if key == QtCore.Qt.Key_Right:
+                if key == QtCore.Qt.Key.Key_Right:
                     self.orbit(azim=-speed, elev=0)
-                elif key == QtCore.Qt.Key_Left:
+                elif key == QtCore.Qt.Key.Key_Left:
                     self.orbit(azim=speed, elev=0)
-                elif key == QtCore.Qt.Key_Up:
+                elif key == QtCore.Qt.Key.Key_Up:
                     self.orbit(azim=0, elev=-speed)
-                elif key == QtCore.Qt.Key_Down:
+                elif key == QtCore.Qt.Key.Key_Down:
                     self.orbit(azim=0, elev=speed)
-                elif key == QtCore.Qt.Key_PageUp:
+                elif key == QtCore.Qt.Key.Key_PageUp:
                     pass
-                elif key == QtCore.Qt.Key_PageDown:
+                elif key == QtCore.Qt.Key.Key_PageDown:
                     pass
                 self.keyTimer.start(16)
         else:
             self.keyTimer.stop()
 
     def checkOpenGLVersion(self, msg):
-        ## Only to be called from within exception handler.
-        ver = glGetString(GL_VERSION).split()[0]
-        if int(ver.split('.')[0]) < 2:
-            from .. import debug
-            pyqtgraph.debug.printExc()
-            raise Exception(msg + " The original exception is printed above; however, pyqtgraph requires OpenGL version 2.0 or greater for many of its 3D features and your OpenGL version is %s. Installing updated display drivers may resolve this issue." % ver)
-        else:
-            raise
-            
+        """
+        Give exception additional context about version support.
 
-            
+        Only to be called from within exception handler.
+        As this check is only performed on error,
+        unsupported versions might still work!
+        """
+
+        # Check for unsupported version
+        verString = glGetString(GL_VERSION)
+        ver = verString.split()[0]
+        # If not OpenGL ES...
+        if str(ver.split(b'.')[0]).isdigit():
+            verNumber = int(ver.split(b'.')[0])
+            # ...and version is supported:
+            if verNumber >= 2:
+                # OpenGL version is fine, raise the original exception
+                raise
+
+        # Print original exception
+        from .. import debug
+        debug.printExc()
+
+        # Notify about unsupported version
+        raise Exception(
+            msg + "\n" + \
+            "pyqtgraph.opengl: Requires >= OpenGL 2.0 (not ES); Found %s" % verString
+        )
+ 
     def readQImage(self):
         """
         Read the current buffer pixels out as a QImage.
@@ -394,16 +559,11 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         pixels[...,3] = 255
         
         glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, pixels)
-        
-        # swap B,R channels for Qt
-        tmp = pixels[...,0].copy()
-        pixels[...,0] = pixels[...,2]
-        pixels[...,2] = tmp
-        pixels = pixels[::-1] # flip vertical
-        
-        img = fn.makeQImage(pixels, transpose=False)
-        return img
-        
+
+        pixels = pixels[::-1].copy() # flip vertical
+
+        qimg = fn.ndarray_to_qimage(pixels, QtGui.QImage.Format.Format_RGBA8888)
+        return qimg
         
     def renderToArray(self, size, format=GL_BGRA, type=GL_UNSIGNED_BYTE, textureSize=1024, padding=256):
         w,h = map(int, size)
@@ -411,6 +571,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
         self.makeCurrent()
         tex = None
         fb = None
+        depth_buf = None
         try:
             output = np.empty((w, h, 4), dtype=np.ubyte)
             fb = glfbo.glGenFramebuffers(1)
@@ -428,8 +589,14 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                 raise Exception("OpenGL failed to create 2D texture (%dx%d); too large for this hardware." % shape[:2])
             ## create teture
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, texwidth, texwidth, 0, GL_RGBA, GL_UNSIGNED_BYTE, data.transpose((1,0,2)))
-            
-            self.opts['viewport'] = (0, 0, w, h)  # viewport is the complete image; this ensures that paintGL(region=...) 
+
+            # Create depth buffer
+            depth_buf = glGenRenderbuffers(1)
+            glBindRenderbuffer(GL_RENDERBUFFER, depth_buf)
+            glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, texwidth, texwidth)
+            glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depth_buf)
+
+            self.opts['viewport'] = (0, 0, w, h)  # viewport is the complete image; this ensures that paintGL(region=...)
                                                   # is interpreted correctly.
             p2 = 2 * padding
             for x in range(-padding, w-padding, texwidth-p2):
@@ -443,6 +610,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                     glfbo.glFramebufferTexture2D(glfbo.GL_FRAMEBUFFER, glfbo.GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, tex, 0)
                     
                     self.paintGL(region=(x, h-y-h2, w2, h2), viewport=(0, 0, w2, h2))  # only render sub-region
+                    glBindTexture(GL_TEXTURE_2D, tex) # fixes issue #366
                     
                     ## read texture back to array
                     data = glGetTexImage(GL_TEXTURE_2D, 0, format, type)
@@ -457,8 +625,7 @@ class GLViewWidget(QtOpenGL.QGLWidget):
                 glDeleteTextures([tex])
             if fb is not None:
                 glfbo.glDeleteFramebuffers([fb])
-            
+            if depth_buf is not None:
+                glDeleteRenderbuffers(1, [depth_buf])
+
         return output
-        
-        
-        
