@@ -23,8 +23,8 @@ class ColorButton(QtGui.QPushButton):
         QtGui.QPushButton.__init__(self, parent)
         self.setColor(color)
         self.colorDialog = QtGui.QColorDialog()
-        self.colorDialog.setOption(QtGui.QColorDialog.ShowAlphaChannel, True)
-        self.colorDialog.setOption(QtGui.QColorDialog.DontUseNativeDialog, True)
+        self.colorDialog.setOption(QtGui.QColorDialog.ColorDialogOption.ShowAlphaChannel, True)
+        self.colorDialog.setOption(QtGui.QColorDialog.ColorDialogOption.DontUseNativeDialog, True)
         self.colorDialog.currentColorChanged.connect(self.dialogColorChanged)
         self.colorDialog.rejected.connect(self.colorRejected)
         self.colorDialog.colorSelected.connect(self.colorSelected)
@@ -35,13 +35,13 @@ class ColorButton(QtGui.QPushButton):
         self.setMinimumWidth(15)
         
     def paintEvent(self, ev):
-        QtGui.QPushButton.paintEvent(self, ev)
+        super().paintEvent(ev)
         p = QtGui.QPainter(self)
         rect = self.rect().adjusted(6, 6, -6, -6)
         ## draw white base, then texture for indicating transparency, then actual color
         p.setBrush(functions.mkBrush('w'))
         p.drawRect(rect)
-        p.setBrush(QtGui.QBrush(QtCore.Qt.DiagCrossPattern))
+        p.setBrush(QtGui.QBrush(QtCore.Qt.BrushStyle.DiagCrossPattern))
         p.drawRect(rect)
         p.setBrush(functions.mkBrush(self._color))
         p.drawRect(rect)
@@ -50,11 +50,11 @@ class ColorButton(QtGui.QPushButton):
     def setColor(self, color, finished=True):
         """Sets the button's color and emits both sigColorChanged and sigColorChanging."""
         self._color = functions.mkColor(color)
+        self.update()
         if finished:
             self.sigColorChanged.emit(self)
         else:
             self.sigColorChanging.emit(self)
-        self.update()
         
     def selectColor(self):
         self.origColor = self.color()
