@@ -66,7 +66,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
         self.setMinimumWidth(0)
         self._lastFontHeight = None
         
-        self.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
+        self.setSizePolicy(QtGui.QSizePolicy.Policy.Expanding, QtGui.QSizePolicy.Policy.Preferred)
         self.errorBox = ErrorBox(self.lineEdit())
         
         self.opts = {
@@ -77,7 +77,6 @@ class SpinBox(QtGui.QAbstractSpinBox):
             'step': D('0.01'),  ## if 'dec' is false, the spinBox steps by 'step' every time
                                 ## if 'dec' is True, the step size is relative to the value
                                 ## 'step' needs to be an integral divisor of ten, ie 'step'*n=10 for some integer value of n (but only if dec is True)
-            'log': False,   # deprecated
             'dec': False,   ## if true, does decimal stepping. ie from 1-10 it steps by 'step', from 10 to 100 it steps by 10*'step', etc. 
                             ## if true, minStep must be set in order to cross zero.
             
@@ -105,7 +104,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
         self.val = D(asUnicode(value))  ## Value is precise decimal. Ordinary math not allowed.
         self.updateText()
         self.skipValidate = False
-        self.setCorrectionMode(self.CorrectToPreviousValue)
+        self.setCorrectionMode(self.CorrectionMode.CorrectToPreviousValue)
         self.setKeyboardTracking(False)
         self.proxy = SignalProxy(self.sigValueChanging, slot=self.delayedChange, delay=self.opts['delay'])
         self.setOpts(**kwargs)
@@ -113,12 +112,6 @@ class SpinBox(QtGui.QAbstractSpinBox):
         
         self.editingFinished.connect(self.editingFinishedEvent)
 
-    def event(self, ev):
-        ret = QtGui.QAbstractSpinBox.event(self, ev)
-        if ev.type() == QtCore.QEvent.KeyPress and ev.key() == QtCore.Qt.Key_Return:
-            ret = True  ## For some reason, spinbox pretends to ignore return key press
-        return ret
-        
     def setOpts(self, **opts):
         """Set options affecting the behavior of the SpinBox.
         
@@ -132,8 +125,10 @@ class SpinBox(QtGui.QAbstractSpinBox):
         siPrefix       (bool) If True, then an SI prefix is automatically prepended
                        to the units and the value is scaled accordingly. For example,
                        if value=0.003 and suffix='V', then the SpinBox will display
-                       "300 mV" (but a call to SpinBox.value will still return 0.003). Default
-                       is False.
+                       "300 mV" (but a call to SpinBox.value will still return 0.003). In case
+                       the value represents a dimensionless quantity that might span many
+                       orders of magnitude, such as a Reynolds number, an SI
+                       prefix is allowed with no suffix. Default is False.
         step           (float) The size of a single step. This is used when clicking the up/
                        down arrows, when rolling the mouse wheel, or when pressing 
                        keyboard arrows while the widget has keyboard focus. Note that
@@ -225,6 +220,13 @@ class SpinBox(QtGui.QAbstractSpinBox):
                 if ms < 1:
                     ms = 1
                 self.opts['minStep'] = ms
+
+            if 'format' not in opts:
+                self.opts['format'] = asUnicode("{value:d}{suffixGap}{suffix}")
+
+        if self.opts['dec']:
+            if self.opts.get('minStep') is None:
+                self.opts['minStep'] = self.opts['step']
         
         if 'delay' in opts:
             self.proxy.setDelay(opts['delay'])
@@ -360,10 +362,9 @@ class SpinBox(QtGui.QAbstractSpinBox):
         if not isinstance(value, D):
             value = D(asUnicode(value))
 
-        changed = value != self.val
-        prev = self.val
-        
-        self.val = value
+        prev, self.val = self.val, value
+        changed = not fn.eq(value, prev)  # use fn.eq to handle nan
+
         if update and (changed or not bounded):
             self.updateText(prev=prev)
 
@@ -381,7 +382,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
     
     def delayedChange(self):
         try:
-            if self.val != self.lastValEmitted:
+            if not fn.eq(self.val, self.lastValEmitted):  # use fn.eq to handle nan
                 self.emitChanged()
         except RuntimeError:
             pass  ## This can happen if we try to handle a delayed signal after someone else has already deleted the underlying C++ object.
@@ -393,7 +394,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
         return QtCore.QSize(120, 0)
     
     def stepEnabled(self):
-        return self.StepUpEnabled | self.StepDownEnabled        
+        return self.StepEnabledFlag.StepUpEnabled | self.StepEnabledFlag.StepDownEnabled        
     
     def stepBy(self, n):
         if isinf(self.val) or isnan(self.val):
@@ -404,13 +405,6 @@ class SpinBox(QtGui.QAbstractSpinBox):
         val = self.val
         
         for i in range(int(abs(n))):
-            
-            if self.opts['log']:
-                raise Exception("Log mode no longer supported.")
-            #    step = abs(val) * self.opts['step']
-            #    if 'minStep' in self.opts:
-            #        step = max(step, self.opts['minStep'])
-            #    val += step * s
             if self.opts['dec']:
                 if val == 0:
                     step = self.opts['minStep']
@@ -464,7 +458,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
 
         # format the string 
         val = self.value()
-        if self.opts['siPrefix'] is True and len(self.opts['suffix']) > 0:
+        if self.opts['siPrefix'] is True:
             # SI prefix was requested, so scale the value accordingly
 
             if self.val == 0 and prev is not None:
@@ -484,29 +478,29 @@ class SpinBox(QtGui.QAbstractSpinBox):
 
     def validate(self, strn, pos):
         if self.skipValidate:
-            ret = QtGui.QValidator.Acceptable
+            ret = QtGui.QValidator.State.Acceptable
         else:
             try:
                 val = self.interpret()
                 if val is False:
-                    ret = QtGui.QValidator.Intermediate
+                    ret = QtGui.QValidator.State.Intermediate
                 else:
                     if self.valueInRange(val):
                         if not self.opts['delayUntilEditFinished']:
                             self.setValue(val, update=False)
-                        ret = QtGui.QValidator.Acceptable
+                        ret = QtGui.QValidator.State.Acceptable
                     else:
-                        ret = QtGui.QValidator.Intermediate
+                        ret = QtGui.QValidator.State.Intermediate
                         
             except:
                 import sys
                 sys.excepthook(*sys.exc_info())
-                ret = QtGui.QValidator.Intermediate
+                ret = QtGui.QValidator.State.Intermediate
             
         ## draw / clear border
-        if ret == QtGui.QValidator.Intermediate:
+        if ret == QtGui.QValidator.State.Intermediate:
             self.textValid = False
-        elif ret == QtGui.QValidator.Acceptable:
+        elif ret == QtGui.QValidator.State.Acceptable:
             self.textValid = True
         ## note: if text is invalid, we don't change the textValid flag 
         ## since the text will be forced to its previous state anyway
@@ -543,7 +537,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
             return False
             
         # check suffix
-        if suffix != self.opts['suffix'] or (suffix == '' and siprefix != ''):
+        if suffix != self.opts['suffix']:
             return False
            
         # generate value
@@ -596,7 +590,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
 
     def paintEvent(self, ev):
         self._updateHeight()
-        QtGui.QAbstractSpinBox.paintEvent(self, ev)
+        super().paintEvent(ev)
 
 
 class ErrorBox(QtGui.QWidget):
@@ -606,12 +600,12 @@ class ErrorBox(QtGui.QWidget):
     def __init__(self, parent):
         QtGui.QWidget.__init__(self, parent)
         parent.installEventFilter(self)
-        self.setAttribute(QtCore.Qt.WA_TransparentForMouseEvents)
+        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_TransparentForMouseEvents)
         self._resize()
         self.setVisible(False)
         
     def eventFilter(self, obj, ev):
-        if ev.type() == QtCore.QEvent.Resize:
+        if ev.type() == QtCore.QEvent.Type.Resize:
             self._resize()
         return False
 
